/*
-- Copyright INMOS Limited 1991
--
-- **************************************************************************
--
-- Group:	Iq software
--
-- Product:	IMS-F006 Ethernet TRAM support software
--
-- File:	b431io.c
--
-- Version:	1.0
--
-- **************************************************************************
--
-- B431 device driver interface library. This file contains procedures for
-- interacting with an IMS-B431 or IMS-B407 ethernet TRAM when executing the
-- ethernet TRAM device driver (b431drvr).
--
-- **************************************************************************
--
-- History:	
--
-- (24-SEP-91) Bob Wipfel, Created file.
--
-- **************************************************************************
*/

#include <string.h>
#include <process.h>
#include <channel.h>

#include "b431io.h"
#include "b431drvr.h"


int B431_Init_Normal(
	Channel		*from_b431,
	Channel		*to_b431,
	const unsigned char	physical_address[PHYSICAL_ADDRESS_SIZE],
	const unsigned char	logical_address_filter[LOGICAL_ADDRESS_FILTER_SIZE],
	const long	mode_flags )
{
	char		message[ PHYSICAL_ADDRESS_SIZE + 
				LOGICAL_ADDRESS_FILTER_SIZE + sizeof(long) ];
	MESSAGE_HDR	header;
	char		*m = message;

	INIT_MESSAGE_HDR( header, sizeof(message), FN_INIT_NORMAL, 0 );

	memcpy( m, physical_address, PHYSICAL_ADDRESS_SIZE );
	m += PHYSICAL_ADDRESS_SIZE;
	memcpy( m, logical_address_filter, LOGICAL_ADDRESS_FILTER_SIZE );
	m += LOGICAL_ADDRESS_FILTER_SIZE;
	memcpy( m, &mode_flags, sizeof(long) );

	ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );
	ChanOut( to_b431, message, header.length );

	ChanIn( from_b431, &header, sizeof(MESSAGE_HDR) );

	return( header.result );
}


int B431_Init_Loopback(
	Channel		*from_b431,
	Channel		*to_b431,
	const unsigned char	physical_address[PHYSICAL_ADDRESS_SIZE],
	const long	mode_flags )
{
	char		message[ PHYSICAL_ADDRESS_SIZE + sizeof(long) ];
	MESSAGE_HDR	header;
	char		*m = message;

	INIT_MESSAGE_HDR( header, sizeof(message), FN_INIT_LOOPBACK, 0 );

	memcpy( m, physical_address, PHYSICAL_ADDRESS_SIZE );
	m += PHYSICAL_ADDRESS_SIZE;
	memcpy( m, &mode_flags, sizeof(long) );

	ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );
	ChanOut( to_b431, message, header.length );

	ChanIn( from_b431, &header, sizeof(MESSAGE_HDR) );
    
	return( header.result );
}


int B431_Start_Ether(
	Channel		*from_b431,
	Channel		*to_b431 )
{
	MESSAGE_HDR header;

	INIT_MESSAGE_HDR( header, 0, FN_START_ETHER, 0 );

	ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );

	ChanIn( from_b431, &header, sizeof(MESSAGE_HDR) );

	return( header.result );
}


void B431_Tx_Packet1(
	Channel		*to_b431,
	const unsigned char	*ethernet_packet,
	const int	ethernet_packet_length )
{
	if ( ethernet_packet_length != 0 )
	{
		MESSAGE_HDR	header;

		INIT_MESSAGE_HDR( header, ethernet_packet_length,
			FN_TX_PACKET1, 0 );

		ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );
		ChanOut( to_b431, (void *) ethernet_packet, header.length );
	}
}		


void B431_Tx_Packet2(
	Channel		*to_b431,
	const unsigned char	destination_address[PHYSICAL_ADDRESS_SIZE],
	const short	type_length_field,
	const unsigned char	*packet_data,
	const int	packet_data_length )
{
	char		message[ PHYSICAL_ADDRESS_SIZE + sizeof(short) ];
	MESSAGE_HDR	header;
	char		*m = message;

	INIT_MESSAGE_HDR( header, sizeof(message), FN_TX_PACKET2, 0 );

	memcpy( m, destination_address, PHYSICAL_ADDRESS_SIZE );
	m += PHYSICAL_ADDRESS_SIZE;
	memcpy( m, &type_length_field, sizeof(short) );
	
	ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );
	ChanOut( to_b431, message, header.length );

	header.length = packet_data_length;

	ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );

	if ( header.length != 0 )
		ChanOut( to_b431, (void *) packet_data, header.length );
}


void B431_Reset_Stats( Channel *to_b431 )
{
	MESSAGE_HDR header;

	INIT_MESSAGE_HDR( header, 0, FN_RESET_STATS, 0 );

	ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );
}


void B431_Stop_Ether( Channel *to_b431 )
{
	MESSAGE_HDR header;

	INIT_MESSAGE_HDR( header, 0, FN_STOP_ETHER, 0 );

	ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );
}


void B431_Terminate( Channel *to_b431 )
{
	MESSAGE_HDR header;

	INIT_MESSAGE_HDR( header, 0, FN_TERMINATE, 0 );

	ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );
}


void B431_Ether_Stats( Channel *to_b431 )
{
	MESSAGE_HDR header;

	INIT_MESSAGE_HDR( header, 0, FN_ETHER_STATS, 0 );

	ChanOut( to_b431, &header, sizeof(MESSAGE_HDR) );
}


int B431_Waitfor_Event( 
	Channel		*from_b431,
	Channel		*cancel,
	ETHER_STATS	*ethernet_stats,
	unsigned char	*ethernet_packet,
	int		*ethernet_packet_length,
	int		*error_code,
	unsigned char	*failed_packet_data )
{
	MESSAGE_HDR	header;

	switch ( ProcAlt( from_b431, cancel, NULL ) )
	{
		char	cancel_token;

		case 0:
			ChanIn( from_b431, &header, sizeof(MESSAGE_HDR) );
			break;

		case 1:
			ChanIn( cancel, &cancel_token, sizeof(char) );
			return( B431_WAIT_CANCELLED );
	}

	switch ( header.fn_code )
	{
		case B431_TERMINATE:
		case B431_STOP_ETHER:
			break;

		case B431_ETHER_STATS:
			ChanIn( from_b431, ethernet_stats, header.length );
			break;

		case B431_ERROR_REPORT:
			*error_code = header.result;
			if ( *error_code == ERROR_TX_PACKET_FAILED )
				ChanIn( from_b431, failed_packet_data, header.length );
			break;

		case B431_RX_PACKET:
			ChanIn( from_b431, ethernet_packet, header.length );
			*ethernet_packet_length = header.length;
			break;
	}

	return( header.fn_code );
}
