--{{{  Header
--
-- Copyright INMOS Limited 1991
--
-- **************************************************************************
--
-- Group:	Iq software
--
-- Product:	IMS-F006 Ethernet TRAM support software
--
-- File:	b431io.inc
--
-- Version:	1.0
--
-- **************************************************************************
--
-- Contains constants that define the interface between the B431 device
-- driver and the corresponding client libraries. Users of the client library
-- will need to include this file in order to make the library calls. This
-- contains ethernet related constants, procedure return codes and describes
-- the ethernet statistics vector structure.
--
-- **************************************************************************
--
-- History:	
--
-- (20-SEP-91) Bob Wipfel, Created file.
--
-- **************************************************************************
--
--}}}

-- Physical ethernet address size (in bytes)

VAL PHYSICAL.ADDRESS.SIZE IS 6 :

-- Logical ethernet address filter size (in bytes)

VAL LOGICAL.ADDRESS.FILTER.SIZE IS 8 : 

-- Ethernet packet header size. All packets have a header prepended to
-- the front of the packet data. In IEEE 802.3 ethernet systems it contains
-- the destination address, source address and a 16 bit length field. In
-- Ethernet V2 systems the length field is used as a packet type code. The
-- header is therefore 14 bytes long.

VAL PACKET.HEADER.SIZE IS (2 * PHYSICAL.ADDRESS.SIZE) + 2 :

-- Minimum / Maximum ethernet packet lengths (in bytes). Short packets will
-- be padded out to the minimum length. The values include the 14 byte packet
-- header and the trailing CRC field (4 bytes). User supplied packets should
-- only include a CRC field if the ethernet interface is initialised with CRC
-- generation disabled. Packets supplied by the ethernet interface will always
-- contain the trailing CRC field bytes.

VAL MIN.PACKET.LENGTH IS 64 :
VAL MAX.PACKET.LENGTH IS 1518 :

-- Minimum / Maximum ethernet packet lengths (in bytes) when operating the
-- B431 driver in loopback mode. The values include the 14 byte packet
-- header and the trailing CRC field (4 bytes). User supplied packets should
-- only include a CRC field if the ethernet interface is initialised with CRC
-- generation disabled. Packets supplied by the ethernet interface will always
-- contain the trailing CRC field bytes.

VAL MIN.LOOPBACK.LENGTH IS 18 :
VAL MAX.LOOPBACK.LENGTH IS 36 :  

-- Failed transmit packet length, the B431 driver returns part of a packet it
-- failed to transmit.

VAL FAILED.PACKET.LENGTH IS 64 :

-- Initialisation mode flags. Bit masks are defined to enable optional
-- Ethernet interface functions or to modify the default operating mode.

VAL MEMORY.CHECK      IS #0001 :
VAL PROMISCUOUS.RX    IS #0002 :
VAL MONITOR.HEARTBEAT IS #0004 :
VAL DISABLE.TX.CRC    IS #0008 :
VAL DISABLE.TX.RETRY  IS #0010 :
VAL INTERNAL.LOOPBACK IS #0020 :
VAL FORCE.COLLISION   IS #0040 :

-- Initialisation return codes.

VAL INIT.SUCCESS         IS 0 (BYTE) : 
VAL INIT.NOT.STOPPED     IS 1 (BYTE) :
VAL INIT.HARDWARE.FAILED IS 2 (BYTE) :
VAL INIT.MEMORY.FAULT    IS 3 (BYTE) :

-- Start ethernet return codes.

VAL START.SUCCESS        IS 0 (BYTE) :
VAL START.NO.INIT.DONE   IS 1 (BYTE) :

-- The B431 driver sends asynchronous messages to the client library to
-- indicate the occurence of an internal event, such as the reception of an
-- ethernet packet, or to respond to a previous asynchronous request message.
-- The following codes are defined to indicate which message was received.

-- These messages handshake a previous request. A request message is sent to
-- the B431 driver by calling the appropriate client library procedure.

VAL B431.STOP.ETHER     IS 16 (BYTE) :  -- Interface shut down
VAL B431.TERMINATE      IS 17 (BYTE) :  -- Device driver terminated
VAL B431.ETHER.STATS    IS 18 (BYTE) :  -- Ethernet statistics available

-- The B431 driver sends these messages to indicate the occurence of an
-- internal event condition.

VAL B431.RX.PACKET      IS 19 (BYTE) :  -- Ethernet packet received
VAL B431.ERROR.REPORT   IS 20 (BYTE) :  -- Device driver error report

-- This code is returned by B431.Waitfor.Event() when cancelled. It allows
-- the B431 interface to be polled.

VAL B431.WAIT.CANCELLED IS 21 (BYTE) :

-- The B431.ETHER.STATS message contains ethernet statistics returned by the
-- driver. The statistics are contained in an INT32 vector as follows.

VAL STATS.TX.PACKETS         IS 0 :
VAL STATS.RX.PACKETS         IS 1 :
VAL STATS.FRAMING.ERRORS     IS 2 :
VAL STATS.CRC.ERRORS         IS 3 :
VAL STATS.PACKETS.DROPPED    IS 4 :
VAL STATS.PACKETS.MISSED     IS 5 :
VAL STATS.DEFERRED.TRANSMITS IS 6 :
VAL STATS.LATE.COLLISIONS    IS 7 :
VAL STATS.CARRIER.LOST       IS 8 :
VAL STATS.NO.MORE.RETRIES    IS 9 :
VAL STATS.SINGLE.RETRIES     IS 10 :
VAL STATS.MULTIPLE.RETRIES   IS 11 :
VAL STATS.AVERAGE.TDR.VALUE  IS 12 :

VAL ETHER.STATS.SIZE         IS 13 :

-- The B431.ERROR.REPORT message contains an error code, it will also contain
-- ethernet packet data in the case of ERROR.TX.PACKET.FAILED.

VAL ERROR.NO.ERROR            IS 0 :
VAL ERROR.TX.PACKET.FAILED    IS 1 :
VAL ERROR.HEARTBEAT.STOPPED   IS 2 :
VAL ERROR.TX.BABBLE.FAULT     IS 3 :
VAL ERROR.DMA.REQUEST.LATE    IS 4 :
VAL ERROR.TX.BUFFER.INVALID   IS 5 :
VAL ERROR.TX.BUFFER.UNDERFLOW IS 6 :
